@@ -12,7 +12,7 @@ Dancer::Plugin::Database - easy database connections for Dancer applications
 
 =cut
 
-our $VERSION = '1.11_01';
+our $VERSION = '1.30_01';
 
 my $settings = undef;
 
@@ -55,8 +55,14 @@ register database => sub {
         }
     }
 
+    # To be fork safe and thread safe, use a combination of the PID and TID (if
+    # running with use threads) to make sure no two processes/threads share
+    # handles.  Implementation based on DBIx::Connector by David E. Wheeler.
+    my $pid_tid = $$;
+    $pid_tid .= '_' . threads->tid if $INC{'threads.pm'};
+
     # OK, see if we have a matching handle
-    $handle = $handles{$handle_key} || {};
+    $handle = $handles{$pid_tid}{$handle_key} || {};
     
     if ($handle->{dbh}) {
         if ($conn_details->{connection_check_threshold} &&
@@ -81,7 +87,7 @@ register database => sub {
         # Get a new connection
         if ($handle->{dbh} = _get_connection($conn_details)) {
             $handle->{last_connection_check} = time;
-            $handles{$handle_key} = $handle;
+            $handles{$pid_tid}{$handle_key} = $handle;
             return $handle->{dbh};
         } else {
             return;
@@ -152,6 +158,7 @@ sub _get_connection {
         Dancer::Logger::error(
             "Database connection failed - " . $DBI::errstr
         );
+        return;
     } elsif (exists $settings->{on_connect_do}) {
         for (@{ $settings->{on_connect_do} }) {
             $dbh->do($_) or Dancer::Logger::error(
@@ -160,6 +167,14 @@ sub _get_connection {
         }
     }
 
+    # Indicate whether queries generated by quick_query() etc in
+    # Dancer::Plugin::Database::Handle should be logged or not; this seemed a
+    # little dirty, but DBI's docs encourage it
+    # ("You can stash private data into DBI handles via $h->{private_..._*}..")
+    $dbh->{private_dancer_plugin_database} = {
+        log_queries => $settings->{log_queries} || 0,
+    };
+
     # Re-bless it as a Dancer::Plugin::Database::Handle object, to provide nice
     # extra features:
     return bless $dbh, 'Dancer::Plugin::Database::Handle';
@@ -297,6 +312,7 @@ should be specified as, for example:
                 RaiseError: 1
                 AutoCommit: 1
             on_connect_do: ["SET NAMES 'utf8'", "SET CHARACTER SET 'utf8'" ]
+            log_queries: 1
 
 The C<connection_check_threshold> setting is optional, if not provided, it
 will default to 30 seconds.  If the database keyword was last called more than
@@ -313,6 +329,13 @@ The optional C<on_connect_do> setting is an array of queries which should be
 performed when a connection is established; if given, each query will be
 performed using C<< $dbh->do >>.
 
+The optional C<log_queries> setting enables logging of queries generated by the
+helper functions C<quick_insert> et al in L<Dancer::Plugin::Database::Handle>.
+If you enable it, generated queries will be logged at 'debug' level.  Be aware
+that they will contain the data you're passing to/from the database, so be
+careful not to enable this option in production, where you could inadvertently
+log sensitive information.
+
 If you prefer, you can also supply a pre-crafted DSN using the C<dsn> setting;
 in that case, it will be used as-is, and the driver/database/host settings will 
 be ignored.  This may be useful if you're using some DBI driver which requires 
@@ -406,6 +429,9 @@ Examples:
   # Delete the row with id 42:
   database->quick_delete($table_name, { id => 42 });
 
+  # Fetch all rows from a table (since version 1.30):
+  database->quick_select($table_name, {});
+
 
 =head1 AUTHOR
 
@@ -437,6 +463,12 @@ Christian Sánchez
 
 Michael Stiller
 
+Martin J Evans
+
+Carlos Sosa
+
+Matt S Trout
+
 =head1 BUGS
 
 Please report any bugs or feature requests to C<bug-dancer-plugin-database at rt.cpan.org>, or through
